import java.awt.Color;

import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.*;

/**
* A caret listener that highlight current selected line or graph in a text pane
* To make a text pane jtextpane has this functionality, simply call
* jtextpane.addCaretListener(new CurrentLineHighlighter)
* 
* @author: Alan Moore, Nguyen T.V
* @see: <a href="http://www.velocityreviews.com/forums/t147950-how-can-i-highlight-the-current-row-in-a-jtextarea.html" />
* CurrentLineHightlighter</a> 
* 
*/
public class CurrentLineHighlighter implements CaretListener
{
	static final Color DEFAULT_COLOR = new Color(135,206,250);
	
	private Highlighter.HighlightPainter painter;
	private Object highlight;

	public CurrentLineHighlighter()
	{
		this(null);
	}

	public CurrentLineHighlighter(Color highlightColor)
	{
		Color c = highlightColor != null ? highlightColor : DEFAULT_COLOR;
		painter = new DefaultHighlighter.DefaultHighlightPainter(c);
	}

	public void caretUpdate(CaretEvent evt)
	{
		JTextComponent comp = (JTextComponent)evt.getSource();
		
		if (comp != null && highlight != null)
		{
			comp.getHighlighter().removeHighlight(highlight);
			highlight = null;
		}

		int pos = comp.getCaretPosition();
		Element elem = Utilities.getParagraphElement(comp, pos);
		int start = elem.getStartOffset();
		int end = elem.getEndOffset();
		try
		{
			// add hight light only when there is no selection
			if (comp.getSelectedText() == null)
				highlight = comp.getHighlighter().addHighlight(start, end,
					painter);
		}
		catch (BadLocationException ex)
		{
			ex.printStackTrace();
		}
	}
}