import java.awt.*;
import java.awt.event.*;

import java.io.File;

import java.awt.event.KeyEvent;

import javax.swing.*;

import com.Ostermiller.Syntax.HighlightedDocument;

import att.grappa.*;
/**
 * Make graphical user interface for this application. <br /> 
 * Any changes (add, remove components, ...) to the GUI
 * should go here
 * 
 * @author TRUNG, N.V
 *
 */
public class MainFrame extends JFrame implements ActionListener 
{
	public MainFrame() {
		super("GUI for Software Model Checking");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// initialize source view
		sourceView = new LNTextPane();
		sourceView.setEditable(false);
		sourceView.addCaretListener(new CurrentLineHighlighter());
		sourceView.setBackground(new Color(238, 238, 238));
		
		// add a scroll pane to wrap the source view pane
		svScroll = new JScrollPane(sourceView);
		svScroll.setPreferredSize(new Dimension(400, 500));
		svScroll.setMinimumSize(new Dimension(300, 500));
		svScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		svScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
        //first split separates source browser and 
		// source view
        firstSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        firstSplit.setRightComponent(svScroll);
        firstSplit.setOneTouchExpandable(true);
		
        secondSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, firstSplit, Utils.getComp(450, 500));
        secondSplit.setOneTouchExpandable(true);
		
        // make menu bar
	    setJMenuBar(makeMenuBar());
	    // make and add toolBar
	    makeToolBar();
	    
	    // create the status text area
	    statusArea = new JTextArea();
	    statusArea.setEditable(false);
	    JScrollPane statusSP = new JScrollPane(statusArea);
	    statusSP.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	    statusSP.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	    statusSP.setBorder(BorderFactory.createTitledBorder("Message"));
	    statusSP.setPreferredSize(new Dimension(650, 100));
	    // split main content and message
	    JSplitPane verticalSplit = new JSplitPane(
			    		JSplitPane.VERTICAL_SPLIT, 
			    		secondSplit, 
			    		statusSP);
	    verticalSplit.setOneTouchExpandable(true);
	    verticalSplit.setDividerLocation(0.8);
	    
	    add(verticalSplit);
	    add(toolBar, BorderLayout.NORTH);
	    
	    pack();
	    setLocation(100,100);
	}
	
	private void makeToolBar() {
		toolBar = new JToolBar();
		
		java.net.URL url;
		url = MainFrame.class.getResource("images/open.png");
		open = new JButton(new ImageIcon(url));
		open.setToolTipText("Open");
		
		open.setActionCommand("Open project");
		open.addActionListener(this);

		url = MainFrame.class.getResource("images/close.png");
		close = new JButton(new ImageIcon(url));
		close.setToolTipText("Close");
		close.setEnabled(false);
		
		close.setActionCommand("Close project");
		close.addActionListener(this);
		
		url = MainFrame.class.getResource("images/zoomin.png");
		gZoomIn = new JButton(new ImageIcon(url));
		gZoomIn.setEnabled(false);
		gZoomIn.setActionCommand("Zoom in");
		gZoomIn.setToolTipText("Zoom In");
		gZoomIn.addActionListener(this);
		
		url = MainFrame.class.getResource("images/zoomout.png");
		gZoomOut = new JButton(new ImageIcon(url));
		gZoomOut.setEnabled(false);
		gZoomOut.setActionCommand("Zoom out");
		gZoomOut.setToolTipText("Zoom Out");
		gZoomOut.addActionListener(this);

		url = MainFrame.class.getResource("images/scale.png");
		gScale = new JButton(new ImageIcon(url));
		gScale.setEnabled(false);
		gScale.setActionCommand("Scale");
		gScale.setToolTipText("Scale to Fit");
		gScale.addActionListener(this);

		// create view button and popup menu
		popup = new JPopupMenu("View");
		
	  	url = MainFrame.class.getResource("images/popup.png");
	  	btnView = new JButton(new ImageIcon(url));
	  	btnView.setToolTipText("Select view mode");	  	
	  	btnView.addActionListener(this);
	  	btnView.setActionCommand("View");
	  	btnView.setEnabled(false);
	  	// create a run button
	  	url = MainFrame.class.getResource("images/run.png");
	  	btnRun = new JButton(new ImageIcon(url));
	  	btnRun.setToolTipText("Run this project");
	  	btnRun.addActionListener(this);
	  	btnRun.setActionCommand("Run");
	  	btnRun.setEnabled(false);
	  	
	  	// <remove me later>
	  	remove = new JButton("Open dot");
	  	remove.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
		    	JFileChooser chooser = new JFileChooser();
		    	chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		    	if (chooser.showDialog(null, "Open project")
		    			== JFileChooser.APPROVE_OPTION) {
		    		File root = chooser.getSelectedFile();
					GrappaPanel gp = Utils.getGrappaPanel(root);
					
					JScrollPane jsp = new JScrollPane(gp);
					jsp.setPreferredSize(new Dimension(450, 500));
					
					JDialog sample = new JDialog(new JFrame(), "Dot view");
					sample.setLayout(new BorderLayout());
		    		sample.add(jsp, BorderLayout.CENTER);
		    		
		    		sample.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		    		sample.setAlwaysOnTop(true);
		    		
		    		sample.setLocation(new Point(100, 100));
		    		sample.pack();
		    		sample.setVisible(true);
		    	}
			}
	  	});
	  	// <remove me later />
	  	
		toolBar.add(open);
		toolBar.add(close);
		toolBar.addSeparator();
		toolBar.add(btnRun);
		toolBar.addSeparator();
		toolBar.add(gZoomIn);
		toolBar.add(gZoomOut);
		toolBar.add(gScale);
		toolBar.addSeparator();
		toolBar.add(btnView);
		
		// remove me later
		toolBar.addSeparator();
		toolBar.add(remove);
	}
	
	private JMenuBar makeMenuBar() {
		JMenuBar bar = new JMenuBar();
		bar.add(makeFileMenu());
		bar.add(makeViewMenu());
		bar.add(makeHelpMenu());
		return bar;
	}
	
	// create View mode for many kind of graphs
	private JMenu makeViewMenu() {
		viewMenu = new JMenu("View");
		viewMenu.setMnemonic(KeyEvent.VK_V);
		//viewMenu.setEnabled(false);
		viewMenu.setToolTipText("Select graph view mode");
		return viewMenu;
	}
	
	private JMenu makeHelpMenu() {
		// help menu
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
		
		JMenuItem help = new JMenuItem("Help contents");
		help.setActionCommand("help");
		help.addActionListener(this);
		helpMenu.add(help);
		
		helpMenu.add(new JMenuItem("About"));
		
		return helpMenu;
	}
	
	private JMenu makeFileMenu() {
		fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		
		// create open menu item
		JMenuItem open = new JMenuItem("Open project...");
		open.setToolTipText("Open a new project");
		open.setActionCommand("Open project");
		open.addActionListener(this);
		
		// create close menu item
		JMenuItem close = new JMenuItem("Close project");
		close.setToolTipText("Close the current project");
		close.setActionCommand("Close project");
		close.addActionListener(this);

		// create exit menu item
		JMenuItem exit = new JMenuItem("Exit");
		exit.setMnemonic(KeyEvent.VK_X);
		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		
		fileMenu.add(open);
		fileMenu.add(close);
		fileMenu.addSeparator();
		fileMenu.add(exit);
		
		return fileMenu;
	}

	/**
	 * Print all message to the status are rather 
	 * than to console
	 * @param msg
	 */
	static void printMessage(String msg) {
		statusArea.append(msg + "\n---\n");
		statusArea.setCaretPosition(statusArea.getText().length());
	}
	
	/**
	 * Enable the graph tool bar such as Zoom In, Zoom Out, HandTool
	 * when the panel is opened
	 */
	static void enableGraphToolBar() {
		gZoomIn.setEnabled(true);
		gZoomOut.setEnabled(true);
		gScale.setEnabled(true);
	}
	
	static void disableGraphToolBar() {
		gZoomIn.setEnabled(false);
		gZoomOut.setEnabled(false);
		gScale.setEnabled(false);
	}
	
	/**
	 * Display the source viewer for this File
	 * @param File source : a source fle
	 */
	static void displaySource(File source) {
    	sourceView.setBackground(Color.WHITE);
    	String content = Utils.getText(source);
    	if (!sourceView.getText().equals(content)) {
    		//doc.setHighlightStyle(HighlightedDocument.C_STYLE);
    		sourceView.setText("\n" + content);
    		sourceView.setCaretPosition(0);
    	}	
        sourceView.setBorder(
        	BorderFactory.createTitledBorder(source.getName()));
	}
	
	/**
	 * Display the graph for the dot file f
	 * Set the title of border of the panel that 
	 * holds this graph to title
	 * @param File f : a dot file
	 * @param String title: title for the border of this panel
	 */
	static void displayGraph(File f, String title) {
       	if (gjsp != null)
       		gjsp.setBorder(BorderFactory.createTitledBorder(title));
       	
       	// if this dot is being viewed
       	if (f.equals(dotFile)) 
       		return;
		// save this file
		dotFile = f;
		
       	GrappaPanel gp = 
       		Utils.getGrappaPanel(f);
       	grappaPanel = gp;
       	if (gp == null) // if dot file was not found
       		return;
       	// hand tool is on by default
       	gp.setHandtool(true);
       	gp.setCursor(new Cursor(Cursor.HAND_CURSOR));
       	gjsp = gp.jscrollpane;
       	gjsp.setMinimumSize(new Dimension(200, 500));
       	gjsp.setPreferredSize(new Dimension(450, 500));
       	gjsp.setViewportView(gp);
       	gjsp.setBorder(
       			BorderFactory.createTitledBorder(title));
       	gjsp.setVisible(true);
       	
       	secondSplit.setRightComponent(gjsp);
       	enableGraphToolBar();
	}
	
	private void openProject() {
    	JFileChooser chooser = new JFileChooser();
    	chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    	if (chooser.showDialog(this,"Open project")
    			== JFileChooser.APPROVE_OPTION) {
    		
    		File root = chooser.getSelectedFile();
    		// Utils.runCommand() should run here
    		// to generate dot files for this project
    		ProjectResource.root = root;
    		
    		browsePanel = new TabbedPanel(new TreePanel(), Utils.getComp(190, 500));
            firstSplit.setLeftComponent(browsePanel);
            
            sourceView.setText("");
            sourceView.setBackground(new Color(238, 238, 238));
            
        	// once project opened -> enable close button, run button and view button
        	close.setEnabled(true);
        	btnRun.setEnabled(true);
    	}	
	}
	
	private void closeProject() {
		if (gjsp != null)
			gjsp.setVisible(false);
		if (browsePanel != null) {
			firstSplit.setLeftComponent(null);
		}	
		if (sourceView != null) {
			sourceView.setText("");
			sourceView.setBackground(new Color(238, 238, 238));
			sourceView.setBorder(null);
		}
		// no project now -> disable close button
		close.setEnabled(false);
		statusArea.setText("");
		disableGraphToolBar();
		
		//viewMenu.setEnabled(false);
		viewMenu.removeAll();
		btnRun.setEnabled(false);
		btnView.setEnabled(false);
		projectStarted = false;
		ProjectResource.root = null;
	}

	// display help window
	private void displayHelp() {
		JDialog dialog = new JDialog(this, "Help");
   		dialog.setLayout(new BorderLayout());
   		JTextPane text = new JTextPane();
   		text.setEditable(false);
   		java.net.URL url = MainFrame.class.getResource("help.html");
   		try {
   			text.setPage(url);
   		} catch (Exception e) {
   			System.err.println(e.getMessage());
   		}
   			
   		JScrollPane jscrollPane = new JScrollPane(text);
		jscrollPane.setPreferredSize(new Dimension(600, 600));
   		dialog.add(jscrollPane, BorderLayout.CENTER);

   		dialog.pack();
		dialog.setVisible(true);
		dialog.setAlwaysOnTop(true);
		dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		dialog.setLocation(200, 200);
	}
	
	// add graph view mode to view menu
	private void updateViewMenu() {
		viewMenu.removeAll();
		ProjectResource.getGraphElements();
		for (int i = 0; i < ProjectResource.graphs.length; i++) {
			String graphName = ProjectResource.graphs[i].getAttribute("name");
			JMenuItem item = new JMenuItem(graphName);
			item.setActionCommand(graphName);
			item.addActionListener(this);
			//item.setToolTipText(text)
			viewMenu.add(item);
		}
	}
	// update popup menu
	private void updatePopupMenu() {
		popup = new JPopupMenu();
		ProjectResource.getGraphElements();
		for (int i = 0; i < ProjectResource.graphs.length; i++) {
			String graphName = ProjectResource.graphs[i].getAttribute("name");
			JMenuItem item = new JMenuItem(graphName);
			item.setActionCommand(graphName);
			item.addActionListener(this);
			popup.add(item);
		}
	}

	/**
	 * Change view mode
	 */
	private void changeView(String graphName) {
		graphMode = graphName;
		printMessage("Switched to view mode " + graphName);
		browsePanel.pane.setComponentAt(1, new TreePanel());
		browsePanel.pane.setSelectedIndex(1);
	}
	/**
	 * Set default graph view after the project has been run.
	 * This helps the GUI look better!
	 */
	private void setDefaultGraphView() {
		graphMode = ProjectResource.graphs[0].getAttribute("name");
		printMessage("Default view mode has been set to " + graphMode);
		browsePanel.pane.setComponentAt(1, new TreePanel());
	}
	
	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();
		if (command.equals("Open project"))  
			openProject();
		else if (command.equals("Close project")) 
			closeProject();
		else if (command.equals("Zoom in")) {
   			grappaPanel.setScaleToFit(false);
   			grappaPanel.setScaleToSize(null);
   			grappaPanel.multiplyScaleFactor(1.25);
   			grappaPanel.clearOutline();
   			grappaPanel.repaint();
   		} else if (command.equals("Zoom out")) {
   			grappaPanel.setScaleToFit(false);
   			grappaPanel.setScaleToSize(null);
   			grappaPanel.multiplyScaleFactor(0.8);
   			grappaPanel.clearOutline();
   			grappaPanel.repaint();
   		} else if (command.equals("Scale")) {
   			grappaPanel.setScaleToFit(true);
   			grappaPanel.repaint();
		} 
   		else if (command.equals("help")) {
   			displayHelp();
   		} // view modes
		else if (command.equals("View")) {
			JButton btn = (JButton) event.getSource();
			if (!isPopupOn) {
				popup.show(btn, 0, btn.getHeight());
				isPopupOn = true;
			} else {
				isPopupOn = false;
				popup.setVisible(false);
			}
		} else if (command.equals("Run")) {
			// each project can be run only once
			if (projectStarted == true)
				return;
			
			// run command to create dot files
			if (ProjectResource.run()) {
				projectStarted = true;
				// now update view menu and popup menu
				updateViewMenu();
				updatePopupMenu();
				btnView.setEnabled(true);
				viewMenu.setEnabled(true);
				// set default graph view
				setDefaultGraphView();
				File logFile = ProjectResource.getLogFile();
				if (logFile.exists()) {
					MainFrame.printMessage("========LOG MESSAGE============");
					MainFrame.printMessage(Utils.getText(logFile));
				}
			}	
		} else  { // change view
			changeView(command);
   		}	
	}

	// variable declarations
	static JEditorPane sourceView;
    static JScrollPane gjsp;
    static GrappaPanel grappaPanel;
    static TabbedPanel browsePanel;
    static JSplitPane firstSplit;
    static JSplitPane secondSplit;
    static JScrollPane svScroll;
    static JToolBar toolBar;
    static JButton open, close;
    static JButton gZoomIn, gZoomOut, gScale;
    static JButton btnView, btnRun;
    static JTextArea statusArea;
    static JMenu fileMenu, viewMenu;
    static JPopupMenu popup;
    static boolean isPopupOn = false;
    static String graphMode;
    static File dotFile;
    static boolean projectStarted = false;
    static HighlightedDocument doc;
    // remove me later
    static JButton remove;
    // end of variable declarations
}
