import java.io.File;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


/**
 * Provides some utilities for reading project xml file 
 * and xml file .
 * 
 * @author NGUYEN, T.V
 *
 */
public class ProjectResource {
	/**
	 * Get the graph element that describes graphName
	 * @param graphName
	 * @return
	 */
	private static Element getGraphElementByName(String graphName) {
		for (int i = 0; i < graphs.length; i++) {
			Element graph = graphs[i];
			if (graph.getAttribute("name").equals(graphName))
				return graph;
		}
		return null;
	}
	
	/**
	 * Return a tree of source files that have graph of type graphName
	 * @param graphName
	 * @return
	 */
	public static JTree getTree(String graphName) {
		Element graph = getGraphElementByName(graphName);
        DefaultMutableTreeNode top =
            new DefaultMutableTreeNode(graphName);
		if (graph.getAttribute("type").equals("1")) { // one source file -> many dots
			NodeList sources = graph.getElementsByTagName("source");
			for (int i = 0; i < sources.getLength(); i++) {
				Element sourceElement = (Element) sources.item(i);
				String pathToSource = sourceElement.getAttribute("pathToSource");
				String pathToDot = sourceElement.getAttribute("pathToDot");
				DefaultMutableTreeNode sourceNode = 
					new DefaultMutableTreeNode(new SourceNodeInfo(pathToSource, pathToDot));
				top.add(sourceNode);
				// now add all function node to this source node
				// <function name="x" pathToDot="x">
				NodeList funcs = sourceElement.getElementsByTagName("function");
				for (int j = 0; j < funcs.getLength(); j++) {
					Element funcElement = (Element) funcs.item(j);
					String name = funcElement.getAttribute("name");
					pathToDot = funcElement.getAttribute("pathToDot");
					DefaultMutableTreeNode funcNode = 
						new DefaultMutableTreeNode(new FuncNodeInfo(name, pathToDot));
					sourceNode.add(funcNode);
				}
			}
		} else if (graph.getAttribute("type").equals("2")) { // one dot -> many sources
			NodeList dots = graph.getElementsByTagName("dot");
			for (int i = 0; i < dots.getLength(); i++) {
				// get a dot element
				Element dotElement = (Element) dots.item(i);
				String pathToDot = dotElement.getAttribute("pathToDot");
				
				DefaultMutableTreeNode dotNode = 
					new DefaultMutableTreeNode(new DotNodeInfo(pathToDot));
				top.add(dotNode);
				// now add all source node to this dot node
				// <source pathToSource="/home/trung/tests/4.c" />
				NodeList srcs = dotElement.getElementsByTagName("source");
				for (int j = 0; j < srcs.getLength(); j++) {
					Element srcElement = (Element) srcs.item(j);
					String pathToSource = srcElement.getAttribute("pathToSource");
					DefaultMutableTreeNode srcNode = 
						new DefaultMutableTreeNode(new SourceNodeInfo(pathToSource, pathToDot));
					dotNode.add(srcNode);
				}
			}
		} else {
			// for a potential graph type that hasn't come yet!
		}
		return new JTree(top);
	}
	
	/**
	 * Return a tree display this project's sources
	 * @return
	 */
	public static JTree getTree() {
		File root = ProjectResource.root;
		DefaultMutableTreeNode top = new DefaultMutableTreeNode(root.getName());
		
		srcs = Utils.getSources();
		for (int i = 0; i < srcs.length; i++)
			top.add(new DefaultMutableTreeNode(new SourceNodeInfo(srcs[i], "")));
		
		JTree tree = new JTree(top);
		return tree;
	}
	
	/**
	 * Read result.xml file and get all graph elements
	 */
	public static void getGraphElements() {
		Document doc = XMLReader.getDocument(root + "/" + GUIConstants.RESULT_FILE);
		NodeList graphElements = doc.getElementsByTagName("graph");
		graphs = new Element[graphElements.getLength()];
		for (int i = 0; i < graphElements.getLength(); i++) {
			graphs[i] = (Element) graphElements.item(i);
		}
	}
	/**
	 * Run clap.byte *.c to generate dot files
	 * @return true if project was run successfully, else if failed
	 */
	public static boolean run() {
        try {
            Runtime rt = Runtime.getRuntime();
            Process pr; 
    		for (int i = 0; i < srcs.length; i++) {
    			pr = rt.exec("clap.byte --skip-main" + srcs[i]);
                int exitVal = pr.waitFor();
                if (exitVal != 0)
                	MainFrame.printMessage("Command: clap.byte --skip-main " + srcs[i] + " failed..");
    		}
        } catch(Exception ex) {
            MainFrame.printMessage(ex.getMessage());
        }
        return true;
	}
	/**
	 * Get path to log file and return it as a file
	 * @return
	 */
	public static File getLogFile() {
		Document doc = XMLReader.getDocument(root + "/" + GUIConstants.RESULT_FILE);
		Element log = (Element) doc.getElementsByTagName("log").item(0);
		String path = log.getAttribute("pathToLog");
		return new File(path);
	}
	
	// path to project file
	static File root;
	static Element[] graphs;
	static String[] srcs;
}
