(* parser/cattr.ml 2009-01-26 *)
(* Copyright (C) 2007-2009 Yonezawa Lab, The University of Tokyo *)

(* GCC Attribute Handler *)

open Cexpr

let string_equal = Misc.string_equal

(* Initializes {attribute_handlers} not to warn about known
   attributes. *)

let init_attribute_null_handler () : unit = (
  (List.iter
      (fun attrname ->
	(Hashtbl.add attribute_handlers attrname
	    attribute_null_handler) ;
	(Hashtbl.add attribute_handlers ("__" ^ attrname ^ "__")
	    attribute_null_handler))
      ["noreturn" ; "nothrow"]))

(* Checks about existence of a simple keyword only (no arguments)
   attribute. *)

let attribute_check_keyword_only_attribute (name0 : string) (l : gcc_attribute_t list) : bool = (
  let name1 = ("__" ^ name0 ^ "__") in
  (List.exists
      (fun a ->
	match a with
	  (ATTR ((k : string), _)) ->
	    ((string_equal k name0) || (string_equal k name1)))
      l))

let attribute_check_noreturn (tn : typenode_t) : bool = (
  let (tx, (attrs : gcc_attribute_t list)) = (strip_off_attributes tn) in
  (attribute_check_keyword_only_attribute "noreturn" attrs))

let attribute_check_nothrow (tn : typenode_t) : bool = (
  let (tx, (attrs : gcc_attribute_t list)) = (strip_off_attributes tn) in
  (attribute_check_keyword_only_attribute "nothrow" attrs))

let attribute_check_nothrow (tn : typenode_t) : bool = (
  let (tx, (attrs : gcc_attribute_t list)) = (strip_off_attributes tn) in
  (attribute_check_keyword_only_attribute "nothrow" attrs))

let attribute_check_in_declaration (key : string) (decl : declaration_t) : bool = (
  let tn = decl.decl_tn in
  let (tx, (attrs : gcc_attribute_t list)) = (strip_off_attributes tn) in
  (attribute_check_keyword_only_attribute key attrs))
