(* parser/pprinters.ml 2009-01-26 *)
(* Copyright (C) 2007-2009 Yonezawa Lab, The University of Tokyo *)

(* Printers for toplevel and debugger. *)

module Sat = Minisat
let string_equal = (Pervasives.(=))

let cube_printer (f : Format.formatter) (cube : Form.cube_t) : unit = (
  (Format.fprintf f "#<cube %s>" (Form.cube_to_string cube)))

let form_printer (f : Format.formatter) (form : Form.form_t) = (
  (Format.fprintf f "#<form %s>" (Form.form_to_string form)))

let solver_printer (f : Format.formatter) (s : Sat.solver_t) : unit = (
  (Format.fprintf f "<solver_t[#constrs=%d]>" (Sat.vec_size s.Sat.sol_constrs)))

let hashtbl_printer (f : Format.formatter) (keyprinter : 'a -> string) (valueprinter : 'b -> string) (t : ('a, 'b) Hashtbl.t) : unit = (
  (Format.fprintf f "#<hashtbl%s>"
      (begin
	let s =
	  (Hashtbl.fold
	      (fun k v b ->
		let s = ((keyprinter k) ^ "->" ^ (valueprinter v)) in
		if (string_equal b "") then s else (s ^ ";" ^ b))
	      t "") in
	if (string_equal s "") then s else (" " ^ s)
      end)))

let varid_expr_hashtbl_printer (f : Format.formatter) (t : (Cexpr.varid_t, Cexpr.expr_t) Hashtbl.t) : unit = (
  (hashtbl_printer f Cexpr.varid_to_string Cexpr.expr_to_string t))

let stmt_block_printer (f : Format.formatter) (b : Csyntax.stmt_block_t) : unit = (
  (Format.fprintf f "#<stmt_block %s>"
      (begin (Csyntax.dump_stmt_block b) ; "" end)))
