/* parser/specs/sg.mly */
/* Copyright (C) 2007-2010 Yonezawa Lab, The University of Tokyo */

/* Note:
     + 2010/06/29 : reimplementation in order to accept full C-expressions
          + part (from line #136 to line #438) of the specification parser
          implementation uses the part of the parser of "why", licensed under
          LGPL2 by the following copyright holders.
          
          Copyright (C) 2002-2008
            Romain BARDOU
            Jean-Fran�ois COUCHOT
            Mehdi DOGGUY
            Jean-Christophe FILLI�TRE
            Thierry HUBERT
            Claude MARCH�
            Yannick MOY
            Christine PAULIN
            Yann R�GIS-GIANAS
            Nicolas ROUSSET
            Xavier URBAIN
*/

%{ 
    open Printf 
    open Lexing   				     
    open Scommon
    open Sexpr

    let abs_pos rel_pos = (
        let fn, ln, cnum_from_bol = !Sexpr.current_pos in
        let new_ln = ln + rel_pos.Lexing.pos_lnum - 1 in
        let new_cnum_from_bol = 
            rel_pos.Lexing.pos_cnum - rel_pos.Lexing.pos_bol in
        let new_cnum_from_bol =
            match !Sexpr.entered_newline with
            | true -> new_cnum_from_bol
            | false -> new_cnum_from_bol + cnum_from_bol
        in {
              Lexing.pos_fname = fn;
              Lexing.pos_lnum = new_ln;
              Lexing.pos_bol = rel_pos.Lexing.pos_bol;
              Lexing.pos_cnum = new_cnum_from_bol
            })
        
    let loc () = (abs_pos (symbol_start_pos ()),
                  abs_pos (symbol_end_pos ()))
    let loc_i i = (rhs_start i, rhs_end i)
    let info x = { lexpr_node = x; lexpr_loc = loc () }
    let loc_info loc x = { lexpr_node = x; lexpr_loc = loc }
    let loc_start x = fst x.lexpr_loc
    let loc_end x = snd x.lexpr_loc
    
    let raising_spec_error_use_loc_func (err_message : string) =
    begin
        let loc = fst (loc ()) in
        let pos =
            (loc.Lexing.pos_fname, loc.Lexing.pos_lnum, loc.Lexing.pos_cnum)
        in
        raise (Sexpr.Specs_error (err_message, Some pos))
    end
    
    let is_in_ensure = ref false
%}

%start global_specs
%type <Sexpr.lspec_t> global_specs
%start local_specs
%type <Sexpr.lspec_t> local_specs
%start full_lexpr
%type <Sexpr.lexpr> full_lexpr
%start function_contract
%type <Sexpr.lfunction_contract_t> function_contract

%token <string> ID TYPENAME
%token <bool*string> STRING_LITERAL
%token <string> CONSTI
%token <string> CONSTF
%token LPA RPA COL DOT DOTDOTDOT
%token INT FLOAT LT GT LE GE EQ NE COM ARW EQL
%token EQUI IMPL AMPAMP BARBAR BAN
%token TTRUE TFALSE OLD RESULT
%token VALID UNIQUE DOL
%token QES MNS PLS STR AMP DIV MOD LBK RBK EOF
%token GLOB INVA FOR ASSRT SEM NULL
%token REQRS ENSRS ASSGN LOOP NOTHING
%token READ_REQRS READ_ENSRS WRITE_REQRS WRITE_ENSRS
%token BREAK CONTINUE RETURN
%token LBC RBC
%token GHOST
%token VOID CHAR SIGNED UNSIGNED SHORT LONG DOUBLE STRUCT ENUM UNION
%token BOOL IMAGINARY COMPLEX SKIPPED
%token TYPE BHVIR ASSMS
%token HAT XOR BAR TLD GTGT LTLT
%token SIZEOF
%token RESET
%token WEAK_INV STRONG_INV

%token PROHIBIT
%token BLOCK CONTEXT YES NO HARDINT_CTX SOFTINT_CTX PROCESS_CTX
%token CONST VOLATILE RESTRICT END ISHANDLER
%token ADDR
%token HEAD TAIL 
%token IF THEN ELSE 	/* for future implementation */
%token INITIALIZER RELEASE CONSTANT NOTNULL NOCONTRACT INIT 
/* "/is_constants" = ISCONST, 
   "/is_not_null" =  ISNOTNULL ,  
   "/is_type"  = ISTYPE  */
%token ISCONST ISTYPE EXCEPT

%token ZERO
/* LPA='(' RPA=')' LBC='{' RBC='}' LBK='[' RBK=']' */
/* AMP='&' STR='*' DOT='.' ARW='->' */
/* EQL='=' QES='?' COL=':' COM=',' SEM=';' ATM='@' DOL='$' */
/* PLS='+' MNS='-' DIV='/' MOD='%' BAR='|' HAT='^' TLD='~' BAN='!' */
/* PLSPLS='++' MNSMNS='--' AMPAMP='&&' BABAR='||' LSH='<<' RSH='>>' */
/* EQUI = '==>'  IMPL = '<==>'*/


%right prec_named
%nonassoc ID TYPENAME
%right QES prec_question
%right IMPL EQUI
%left BARBAR            /* OR */
%left XOR               /* HATHAT */
%left AMPAMP
%left BAR
%left HAT
%nonassoc prec_no_rel
%left prec_rel_list /* for list of relations (LT GT LE GE EQ NE) */
%left LT GT LE GE
%left EQ NEQ
%left LTLT GTGT
%left PLS MNS
%left STR DIV MOD AMP
%right prec_uminus
%right prec_cast
%nonassoc TLD BAN
%left DOT ARW LBK
%right prec_par

%%

enter_acsl_term_mode:
/* empty */ { enter_acsl_term_mode () }

exit_acsl_term_mode:
/* empty */ { exit_acsl_term_mode () }

/*** predicates and terms ***/

ne_lexpr_list:
| lexpr                    { [$1] }
| lexpr COM ne_lexpr_list { $1 :: $3 }
;

lexpr_rel:
| lexpr_rel1 %prec prec_no_rel { $1 }
| lexpr_rel EQ lexpr_rel1 { info (PLrel ($1, Eq, $3)) }
| lexpr_rel NE lexpr_rel1 { info (PLrel ($1, Neq, $3)) }
| ZERO LPA STR lexpr_rel1 RPA {
    if (!is_in_ensure = false) then
        raising_spec_error_use_loc_func Sexpr.zero_error_message
    else
        info (PLrel (info (PLfield $4),
                     Eq,
                     info (Sexpr.PLconstant (Sexpr.IntConstant "0"))))
}
;

lexpr_rel1:
| lexpr_inner %prec prec_no_rel { $1 }
| lexpr_rel1 LT lexpr_rel1 { info (PLrel ($1, Lt, $3)) }
| lexpr_rel1 GT lexpr_rel1 { info (PLrel ($1, Gt, $3)) }
| lexpr_rel1 LE lexpr_rel1 { info (PLrel ($1, Le, $3)) }
| lexpr_rel1 GE lexpr_rel1 { info (PLrel ($1, Ge, $3)) }
;

lexpr_inner:
| string {
      let (is_wide,content) = $1 in
      let cst = if is_wide then
        WStringConstant content
      else
        StringConstant content
      in
      info (PLconstant cst)
    }
| BAN lexpr_inner { info (PLnot $2) }
| TTRUE { info PLtrue }
| TFALSE { info PLfalse }
| VALID LPA lexpr RPA { info (PLvalid ($3)) }
| UNIQUE LPA lexpr COM lexpr RPA {
    if (!is_in_ensure = false) then (
        let loc = fst (loc ()) in
        let pos =
            (loc.Lexing.pos_fname, loc.Lexing.pos_lnum, loc.Lexing.pos_cnum)
        in
        raise (Sexpr.Specs_error ("Invalid use of construct $unique", Some pos))
    )
    else info (PLuniqueblock ($3, $5))
}
| UNIQUE LPA lexpr RPA {
    if (!is_in_ensure = false) then (
        let loc = fst (loc ()) in
        let pos =
            (loc.Lexing.pos_fname, loc.Lexing.pos_lnum, loc.Lexing.pos_cnum)
        in
        raise (Sexpr.Specs_error ("Invalid use of construct $unique", Some pos))
    )
    else info (PLuniqueloc ($3))
}
| CONSTI { info (PLconstant (IntConstant $1)) }
| CONSTF { info (PLconstant (FloatConstant $1)) }
| lexpr_inner PLS lexpr_inner { info (PLbinop ($1, Badd, $3)) }
| lexpr_inner MNS lexpr_inner { info (PLbinop ($1, Bsub, $3)) }
| lexpr_inner STR lexpr_inner { info (PLbinop ($1, Bmul, $3)) }
| lexpr_inner DIV lexpr_inner { info (PLbinop ($1, Bdiv, $3)) }
| lexpr_inner MOD lexpr_inner { info (PLbinop ($1, Bmod, $3)) }
| lexpr_inner ARW identifier { info (PLarrow ($1, $3)) }
| lexpr_inner DOT identifier { info (PLdot ($1, $3)) }
| lexpr_inner LBK lexpr RBK { info (PLarrget ($1, $3)) }
| MNS lexpr_inner %prec prec_uminus { info (PLunop (Uminus, $2)) }
| PLS lexpr_inner %prec prec_uminus { $2 }
| TLD lexpr_inner { info (PLunop (Ubw_not, $2)) }
| STR lexpr_inner { info (PLunop (Ustar, $2)) }
| AMP lexpr_inner { info (PLunop (Uamp, $2)) }
| SIZEOF LPA lexpr RPA { info (PLsizeofE $3) }
| SIZEOF LPA logic_type RPA { info (PLsizeof $3) }
| OLD LPA lexpr RPA {
    try (
        Sexpr.visit_lexpr $3 (Sexpr.check_old_in_old);
        info (PLold $3))
    with
    | Sexpr.Specs_error (msg, _)
      when ((String.compare msg Sexpr.old_in_old_error_msg) = 0) ->
        let loc = fst (loc ()) in
        let pos =
            (loc.Lexing.pos_fname, loc.Lexing.pos_lnum, loc.Lexing.pos_cnum)
        in
        raise (Sexpr.Specs_error ("Expressions of the form $old(...$old(...)...) are forbidden", Some pos))
    | e ->
        raise e }
| RESULT { info PLresult }
| identifier LPA ne_lexpr_list RPA
    { info (PLapp ($1, $3)) }
| identifier LPA RPA
    { info (PLapp ($1, [])) }
| identifier %prec ID { info (PLvar $1) }
| lexpr_inner GTGT lexpr_inner { info (PLbinop ($1, Brshift, $3))}
| lexpr_inner LTLT lexpr_inner { info (PLbinop ($1, Blshift, $3))}
| LPA lexpr RPA %prec prec_par { info $2.lexpr_node }
| LPA logic_type RPA lexpr_inner %prec prec_cast
    { info (PLcast ($2, $4)) }
;

string:
| STRING_LITERAL { $1 }
| string STRING_LITERAL {
      let (is_wide,prefix) = $1 in
      let (is_wide2,suffix) = $2 in
      (is_wide || is_wide2, prefix ^ suffix)
    }

identifier:
| ID { $1 }
;

lexpr:
  /* predicates */
| lexpr IMPL lexpr { info (PLimplies ($1, $3)) }
| lexpr EQUI lexpr { info (PLiff ($1, $3)) }
| lexpr BARBAR lexpr     { info (PLor ($1, $3)) }
| lexpr AMPAMP lexpr    { info (PLand ($1, $3)) }
| lexpr XOR lexpr    { info (PLxor ($1, $3)) }
/* terms */
| lexpr AMP lexpr { info (PLbinop ($1, Bbw_and, $3)) }
| lexpr BAR lexpr { info (PLbinop ($1, Bbw_or, $3)) }
| lexpr HAT lexpr { info (PLbinop ($1, Bbw_xor, $3)) }
| lexpr QES lexpr COL lexpr %prec prec_question
    { info (PLif ($1, Some $3, $5)) }
| lexpr QES COL lexpr %prec prec_question
    { info (PLif ($1, None, $4)) }
/* both terms and predicates */
| lexpr_rel %prec prec_rel_list { $1 }
;

lexpr_option:
| /* epsilon */ { None }
| lexpr         { Some $1 }
;

var_spec:
|       var_spec_bis { $1 }
| stars var_spec_bis
  { let (modif, name) = $2 in
      ((fun x -> $1 (modif x)), name) }
;

var_spec_bis:
| identifier     { ((fun x -> x), $1) }
| var_spec_bis LBK lexpr_option RBK
      { (* TODO: use size information for LTarray - $3 *)
        let (modif, name) = $1 in
          ((fun x -> LTarray (modif x)), name)
      }
| LPA var_spec RPA { $2 }
| var_spec_bis LPA abs_param_type_list RPA { (* TODO *) raise Parse_error }
;

abs_param_type_list:
| /* empty */ { [ ] }
| abs_param_list { $1 }
| abs_param_list COM DOTDOTDOT { (* TODO *) raise Parse_error }
;

abs_param_list:
| abs_param { [ $1 ] }
| abs_param_list COM abs_param { $1 @ [ $3 ] }
;

/* TODO: abs_param should be less restrictive than parameter
since its name can be omitted
*/
abs_param:
| parameter { $1 }
;

/*** restricted type expressions ***/

id_as_typename:
  identifier { LTnamed $1 }
;

parameter:
| type_spec var_spec { let (modif, name) = $2 in (modif $1, name)}
| id_as_typename var_spec { let (modif, name) = $2 in (modif $1, name) }
;

/*** type expressions ***/

logic_type:
| type_spec abs_spec_option { $2 $1 }
;

abs_spec_option:
| /* empty */ %prec TYPENAME  { fun t -> t }
| abs_spec { $1 }
;

abs_spec:
|                    tabs { $1 }
| stars                   %prec TYPENAME { $1 }
| stars              tabs                { fun t -> $2 ($1 t) }
| stars abs_spec_bis      %prec TYPENAME { fun t -> $2 ($1 t) }
| stars abs_spec_bis tabs                { fun t -> $2 ($3 ($1 t)) }
|       abs_spec_bis tabs                { fun t -> $1 ($2 t) }
;

abs_spec_bis:
| LPA abs_spec RPA { $2 }
| abs_spec_bis LPA /*abs_param_type_list*/ RPA { (* TODO *) raise Parse_error };
;

stars:
| STR       { fun t -> LTpointer t }
| stars STR { fun t -> $1 (LTpointer t) }
;

tabs:
| LBK lexpr_option RBK %prec TYPENAME
    {  (* TODO: use size information for LTarray - $2 *)
      fun t -> LTarray t
    }
| LBK lexpr_option RBK tabs
    {  (* TODO: use size information for LTarray - $2 *)
      fun t -> LTarray ($4 t)
    }
;

type_spec:
| VOID           { LTvoid }
| CHAR           { LTint IChar }       /** [char] */
| SIGNED CHAR    { LTint ISChar }      /** [signed char] */
| UNSIGNED CHAR  { LTint IUChar }      /** [unsigned char] */
| INT            { LTint IInt }        /** [int] */
| SIGNED INT     { LTint IInt }        /** [int] */
| UNSIGNED INT   { LTint IUInt }       /** [unsigned int] */
| UNSIGNED       { LTint IUInt }
| SHORT          { LTint IShort }      /** [short] */
| SIGNED SHORT   { LTint IShort }      /** [short] */
| UNSIGNED SHORT { LTint IUShort }     /** [unsigned short] */
| LONG           { LTint ILong }       /** [long] */
| SIGNED LONG    { LTint ILong }       /** [long] */
| UNSIGNED LONG  { LTint IULong }      /** [unsigned long] */
| SIGNED LONG INT{ LTint ILong }       /** [long] */
| LONG  INT      { LTint ILong }       /** [long] */
| UNSIGNED LONG INT { LTint IULong }      /** [unsigned long] */
| LONG LONG      { LTint ILongLong }   /** [long long] (or [_int64] on
					   Microsoft Visual C) */
| SIGNED LONG LONG   { LTint ILongLong }   /** [long long] (or [_int64] on
					   Microsoft Visual C) */
| UNSIGNED LONG LONG { LTint IULongLong }  /** [unsigned long long]
                                (or [unsigned _int64] on Microsoft Visual C) */
| LONG LONG INT     { LTint ILongLong }   /** [long long] (or [_int64] on
					   Microsoft Visual C) */
| SIGNED LONG LONG INT  { LTint ILongLong }   /** [long long] (or [_int64] on
					   Microsoft Visual C) */
| UNSIGNED LONG LONG INT { LTint IULongLong }  /** [unsigned long long]
                                (or [unsigned _int64] on Microsoft Visual C) */
| FLOAT             { LTfloat FFloat }
| DOUBLE            { LTfloat FDouble }
| LONG DOUBLE       { LTfloat FLongDouble }
| STRUCT identifier { LTstruct $2 }
| ENUM   identifier { LTenum $2 }
| UNION  identifier  { LTunion $2 }
| TYPENAME          { LTnamed $1 }
;

/*** annotations ***/

full_lexpr:
enter_acsl_term_mode lexpr exit_acsl_term_mode { $2 }
;

full_identifier:
enter_acsl_term_mode identifier exit_acsl_term_mode { $2 }
;

full_assigns:
enter_acsl_term_mode assigns exit_acsl_term_mode { $2 }
;

assigns:
  location_dep { $1 }
| NOTHING { [(Nothing, [])] }
;

location_dep:
| zones { List.map (fun x -> (x, [])) $1 }
;

zones : tsets { List.map (fun x -> Location x) $1 }

tsets:
| location { [$1] }
| location COM tsets { $1 :: $3 }
;

location:
| lexpr { $1 }
;

ne_tvar_list:
| full_identifier { [$1] }
| full_identifier COM ne_tvar_list { $1 :: $3 }
;

ne_full_identifier_list:
| full_identifier { [$1] }
| full_identifier COM ne_tvar_list { $1 :: $3 }
;

global_specs:
| global_invariant
    { Sexpr.SPEC_GLOBAL_INV $1 }
| type_invariant
    { Sexpr.SPEC_TYPE_INV $1 } 
| function_contract
    { Sexpr.SPEC_FUN $1 }
| var_specs
    { Sexpr.SPEC_VAR $1 }
| error 
    { let start_pos = Parsing.symbol_start_pos () in
      let (current_fn, current_ln, current_cn) = !Sexpr.current_pos in 
      let ln = (current_ln) + start_pos.Lexing.pos_lnum -1 in
      let pos : Sexpr.global_pos_t = (current_fn, ln, -1) in 
      raise (Sexpr.Specs_error ("Specification syntax error", Some pos)) }
;

local_specs:
| statement_annotation_list
    { Sexpr.SPEC_STMT  $1 }
| loop_annotation_list
    { Sexpr.STMT_LOOP $1 }
| var_specs
    { Sexpr.SPEC_VAR $1 }
| error 
    { let start_pos = Parsing.symbol_start_pos () in
      let (current_fn, current_ln, current_cn) = !Sexpr.current_pos in 
      let ln = (current_ln) + start_pos.Lexing.pos_lnum -1 in
      let pos : Sexpr.global_pos_t = (current_fn, ln, -1) in 
      raise (Sexpr.Specs_error ("Specification syntax error", Some pos)) }
;

inv_strength:
| WEAK_INV
    { WEAK_INV }
| STRONG_INV
    { STRONG_INV }
;

global_invariant1:
| GLOB INVA full_identifier EXCEPT LBC except_type_list RBC COL full_lexpr SEM
    { ($3, $9, Some($6)) }
| GLOB INVA full_identifier COL full_lexpr SEM
    { ($3, $5, None) }
;

global_invariant:
| inv_strength global_invariant1
    { let a, b, c = $2 in
      (make_global_invariant $1 a b c) } 
| global_invariant1
    { let a, b, c = $1 in
      (make_global_invariant WEAK_INV a b c) } 
;

type_invariant1: 
| TYPE INVA full_identifier LPA type_spec identifier RPA
    EXCEPT LBC except_type_list RBC 
    EQ full_lexpr SEM
    { ($3, $5 , $6, $13, Some($10)) } 
| TYPE INVA full_identifier LPA type_spec identifier RPA EQ full_lexpr SEM
    { ($3, $5, $6, $9, None) } 
;

type_invariant:
| inv_strength type_invariant1
    { let a, b, c, d, e = $2 in
      (make_type_invariant $1 a b c d e) }
| type_invariant1
    { let a, b, c, d, e = $1 in
      (make_type_invariant WEAK_INV a b c d e) }
;

except_type_list:
| except_type 
    { [$1] } 
| except_type_list COM except_type
    { $1 @ ([$3]) } 

except_type:
| identifier			/* "except" "(" name ")" */ 
    { Sexpr.EXCEPT_WHOLE $1 }
| HEAD LPA identifier RPA	/* "head" "(" name ")" */
    { Sexpr.EXCEPT_HEAD $3 }
| TAIL LPA identifier RPA	/* "tail" "(" name ")" */
    { Sexpr.EXCEPT_TAIL $3 }
;

function_contract: 
| is_handler context
    requires_clause behaviors_or_default
    prohibits local_invariants
    {
        let (default_behaviors, named_behaviors) = $4 in
        let (_, ensures, assigns) = default_behaviors in
        let _ =
            match assigns with
            | [] -> (
                let start_loc = fst (loc ()) in
                warning (start_loc.Lexing.pos_fname
                         ^ ":"
                         ^ (string_of_int start_loc.Lexing.pos_lnum)
                         ^ ": a function contract should have at least one $assigns clause, e.g. '$assigns $nothing;'"))
            | _ -> ()
        in
        let requires = $3 in
        (make_function_contract (requires @ ensures)
                                named_behaviors
                                assigns
                                $5
                                $6
                                $2
                                $1)
    }
;

is_handler:
| /* empty */
    {false}
| ISHANDLER LPA TTRUE RPA SEM
    {true}
| ISHANDLER LPA TFALSE RPA SEM
    {false}

context:
| /* empty */
    {Sexpr.ANY}
| CONTEXT LPA context_content RPA SEM 
    {$3}
;
context_content:
| HARDINT_CTX {Sexpr.HARD}
| SOFTINT_CTX {Sexpr.SOFT}
| PROCESS_CTX {Sexpr.PROCESS}
;

local_invariants:
| /* empty */
    { [] }
| INVA identifier LBC full_lexpr RBC SEM local_invariants
    { ($2, $4) :: $7 }
;

/* prohibit list */
prohibits:
| /* empty */
    { [] } 
| PROHIBIT LBC ne_full_identifier_list RBC SEM
    { $3 }
;

requires_clause :
| /* epsilon */
    { [] }
| REQRS full_lexpr SEM requires_clause
    { (Sexpr.visit_lexpr 
          ($2) 
          (check_validity_of_pre_condition Sexpr.check_old_in_lexpr));
      (Sexpr.visit_lexpr 
          ($2) 
          (check_validity_of_pre_condition Sexpr.check_result_in_lexpr));
      (Sexpr.PRE_CONDITION, $2) :: $4 }
;

var_specs :
| read_requires_clause read_ensures_clause
  write_requires_clause write_ensures_clause
    { ($1 @ $2 @ $3 @ $4) }
;
read_requires_clause :
| /* epsilon */
    { [] }
| READ_REQRS full_lexpr SEM read_requires_clause
    { (Sexpr.READ Sexpr.PRE_CONDITION, $2) :: $4 }
;

read_ensures_clause :
| /* epsilon */
    { [] }
| READ_ENSRS full_lexpr SEM read_ensures_clause
    { ((Sexpr.READ (Sexpr.POST_CONDITION false)), $2) :: $4 }
;

write_requires_clause :
| /* epsilon */
    { [] }
| WRITE_REQRS full_lexpr SEM write_requires_clause
    { (Sexpr.WRITE Sexpr.PRE_CONDITION, $2) :: $4 }
;

write_ensures_clause :
| /* epsilon */
    { [] }
| WRITE_ENSRS full_lexpr SEM write_ensures_clause
    { (Sexpr.WRITE (Sexpr.POST_CONDITION false), $2) :: $4 }
;

behavior_body:
| /* epsilon */ { [],[],[] }
| ne_behavior_body { $1 }

set_is_in_ensure :
| /* empty */ { is_in_ensure := true }

reset_is_in_ensure :
| /* empty */ { is_in_ensure := false }
 
ne_behavior_body:
| ASSMS full_lexpr SEM behavior_body
    { let a,b,c = $4 in $2::a,b,c }
| set_is_in_ensure ENSRS full_lexpr reset_is_in_ensure SEM behavior_body
    { (* check validity of construct $unique in arithmetic expressions *)
      (Sexpr.visit_lexpr $3 
                         (Sexpr.check_validity_of_a_structure
                             Sexpr.check_unique_isnt_in_logic_lexpr
                             Sexpr.unique_error_msg));
      let a,b,c = $6 in a,(POST_CONDITION false, $3)::b,c
    }
| ASSGN full_assigns SEM behavior_body
    { let a,b,c = $4 in
(*       let assigns = *)
(*         match c,$2 with *)
(*             ([] | [(Nothing,_) ]), ([] | [(Nothing,_) ]) -> [(Nothing,[])] *)
(*           | [(Nothing,_) ], _ | _, [(Nothing,_)] -> *)
(*               raise ( *)
(*                 Not_well_formed (loc(),"Mixing \\nothing and a real location")) *)
(*           | _,_ -> $2 @ c *)
      a,b,($2 @ c)
    }

behaviors:
| /* epsilon */ { [] }
| BHVIR full_identifier COL behavior_body behaviors
    { let (assumes,ensures,assigns) = $4 in
      (Sexpr.make_behavior $2 assumes ensures assigns)::$5 }
/*(*         Logic_utils.check_assigns ~loc:(loc()) assigns; *)
(*         {bh_name=$2; bh_assumes = assumes; *)
(*          bh_ensures = ensures; *)
(*          bh_assigns = assigns}::$5 } *)*/
;

behaviors_or_default:
| behavior_body behaviors
    { let (assumes,ensures,assigns) = $1 in
(*       let behaviors = *)
(*         if (*TODO: enforce to have a default behavior true || *) *)
(*            assumes <> [] || ensures <> [] || assigns <> [] then *)
(*           { bh_name = "default";bh_assumes = assumes; *)
(*             bh_ensures = ensures; *)
(*             bh_assigns = assigns} :: $2 *)
(*         else $2 *)
(*       in *)
      if assumes <> [] then
          assert false;		(* disallow assumes clauses in
	                           the default behavior *)
      ($1, $2)
    }
;

statement_annotation_list :
  statement_annotation
    { [$1] }
| statement_annotation_list statement_annotation
    { $1 @ [$2] }
;

statement_annotation :
| assert_clause
    { $1 }
| assume_clause
    { $1 }
| reset_clause
    { $1 }
| init_clause
    { $1 }
| release_clause
    { $1 }
;

assume_clause :
| ASSMS full_lexpr SEM
    { Sexpr.STMT_ASSUME $2 }
;

reset_clause :
| RESET LPA full_assigns RPA SEM
    { Sexpr.STMT_RESET $3 }
;

init_clause:
| INIT LPA ne_full_identifier_list RPA SEM
    { Sexpr.STMT_INIT $3 }
;

release_clause:
| RELEASE LPA ne_full_identifier_list RPA SEM
    { Sexpr.STMT_RELEASE $3 }
;

loop_annotation_list:
| loop_annotation
    { [$1] }
| loop_annotation_list loop_annotation
    { $1 @ [$2] } 
;

loop_annotation:
| loop_clause
    { let ids, expr = $1 in
      Sexpr.STMT_LOOP_INV (ids, expr) }
| break_clause
    { let ids, expr = $1 in
      Sexpr.STMT_BREAK (ids, expr) }
| continue_clause
    { let ids, expr = $1 in
      Sexpr.STMT_CONTINUE (ids, expr) }
| return_clause
    { let ids, expr = $1 in
      Sexpr.STMT_RETURN (ids, expr) }
;

assert_clause:
| ASSRT LPA full_lexpr RPA SEM			
    { Sexpr.STMT_ASSERT ([], $3) }
| FOR ne_full_identifier_list COL ASSRT LPA full_lexpr RPA SEM
    { Sexpr.STMT_ASSERT ($2, $6) }   
;

loop_clause :
| LOOP INVA full_lexpr SEM	               
    { ([], $3) }
| FOR ne_full_identifier_list COL LOOP INVA full_lexpr SEM 
    { ($2, $6) }     
;

break_clause :
| BREAK COL full_lexpr SEM  
    { ([], $3) }
| FOR ne_full_identifier_list COL BREAK COL full_lexpr SEM
    { ($2, $6) }
;

continue_clause :
| CONTINUE COL full_lexpr SEM 
    { ([], $3) }
| FOR ne_full_identifier_list COL CONTINUE COL full_lexpr SEM
    { ($2, $6) }
;

return_clause :
| RETURN COL full_lexpr SEM
    { ([], $3) }
| FOR ne_full_identifier_list COL RETURN COL full_lexpr SEM
    { ($2, $6) } 
;

%%
