(* parser/specs/slex.mll *) 
(* Copyright (C) 2007-2010 Yonezawa Lab, The University of Tokyo *)

(* Note:
     + 2010/06/29 : reimplementation in order to accept full C-expressions
*)

{ 
    open Sg 
    open Lexing
    (* for future use *)
    let boo = ref false
    let str_specal = ref ""		
    let current_comment = ref ""
    let comment_pos : Sexpr.global_pos_t ref = Sexpr.current_pos
    
    let make_local_pos rel_pos abs_ln abs_cn new_line =
        let new_ln = abs_ln + rel_pos.Lexing.pos_lnum - 1 in
        let new_cnum_from_bol =
            rel_pos.Lexing.pos_cnum - rel_pos.Lexing.pos_bol in
        let new_cnum_from_bol =
            match new_line with
            | true -> new_cnum_from_bol
            | false -> new_cnum_from_bol + abs_cn
        in (new_ln, new_cnum_from_bol)

    let reset_space () = 
        let temp = !str_specal in
        str_specal := "";
        temp

    let eat_space lexbuf = (
        let str = Lexing.lexeme lexbuf in 
        str_specal := (!str_specal) ^ str
    )

    (* ======================= *)
    let incr_lineno lexbuf =
      (Sexpr.entered_newline := true) ;
      let pos = lexbuf.lex_curr_p in
      lexbuf.lex_curr_p <- {pos with
              pos_lnum = pos.pos_lnum + 1;
              pos_bol = pos.pos_cnum;}		

    let identifier =
        let all_kw = Hashtbl.create 37 in
        let c_kw = Hashtbl.create 37 in
        List.iter
            (fun (i,t,flag) ->
                Hashtbl.add all_kw i t;
                if flag then Hashtbl.add c_kw i t
            )
            [
                (* C keywords go first *)
                "void", VOID, true;
                "char", CHAR, true;
                "short", SHORT, true;
                "int", INT, true;
                "long", LONG, true;
                "float", FLOAT, true;
                "double", DOUBLE, true;
                "signed", SIGNED, true;
                "unsigned", UNSIGNED, true;
                "_Bool", BOOL, true;
                "_Complex", COMPLEX, true;
                "_Imaginary", IMAGINARY, true;

                "typedef", SKIPPED, true;
                "extern", SKIPPED, true;
                "static", SKIPPED, true;
                "auto", SKIPPED, true;
                "register", SKIPPED, true;
                "inline", SKIPPED, true;

                "const", SKIPPED, true;
                "volatile", SKIPPED, true;
                "restrict", SKIPPED, true;

                "enum", ENUM, true;
                "struct", STRUCT, true;
                "union", UNION, true;

                "break", SKIPPED, true;
                "case", SKIPPED, true;
                "continue", SKIPPED, true;
                "default", SKIPPED, true;
                "do", SKIPPED, true;
                "else", SKIPPED, true;
                "for", SKIPPED, true;
                "zero", SKIPPED, true;
                "goto", SKIPPED, true;
                "if", SKIPPED, true;
                "return", SKIPPED, true;
                "sizeof", SIZEOF, true;
                "switch", SKIPPED, true;
                "while", SKIPPED, true;

                "offsetof", SKIPPED, true; (* macro in C99 *)
                "asm", SKIPPED, true;
                "__asm__", SKIPPED, true;
                "__asm", SKIPPED, true;

                "__Pragma", SKIPPED, true;
                "__func__", SKIPPED, true; (* macro in C99 *)
                "__FUNCTION__", SKIPPED, true;
                "__PRETTY_FUNCTION__", SKIPPED, true;

                "typeof", SKIPPED, true;
                "__typeof__", SKIPPED, true;
                "__typeof", SKIPPED, true;

                "__alignof__", SKIPPED, true;
                "__alignof", SKIPPED, true;

                (* GCC ADDED (See /usr/include/sys/cdefs.h). *)
                
                "__extension__", SKIPPED, true;
                
                "__const__", SKIPPED, true;
                "__const", SKIPPED, true;
                "__inline__", SKIPPED, true;
                "__inline", SKIPPED, true;
                "_inline", SKIPPED, true;
                "__restrict__", SKIPPED, true;
                "__restrict", SKIPPED, true;
                "__signed__", SIGNED, true;
                "__signed", SIGNED, true;
                "__volatile__", SKIPPED, true;
                "__volatile", SKIPPED, true;
       
                (*("__builtin_object_size", Cg.Q_Error, GCC)*)
                "__builtin_va_list", SKIPPED, true;
                "__builtin_offsetof", SKIPPED, true;
                "__builtin_choose_expr", SKIPPED, true;
                "__builtin_types_compatible_p", SKIPPED, true;
                "__builtin_va_arg", SKIPPED, true;

                "__attribute__", SKIPPED, true;
                "__attribute", SKIPPED, true;

                "__deprecated__", SKIPPED, true;
                "__format_arg__", SKIPPED, true;
                "__noinline__", SKIPPED, true;
                "__nonnull__", SKIPPED, true;
                "__nothrow__", SKIPPED, true;
                "__pure__", SKIPPED, true;
                "__strfmon__", SKIPPED, true;
                "__unused__", SKIPPED, true;
                "__used__", SKIPPED, true;

                "__label__", SKIPPED, true;
            ];
        fun s ->           
            try
                Hashtbl.find all_kw s
            with Not_found ->
                if Scommon.typename_status s then TYPENAME s
                else ID s

    let bs_identifier =
        let all_acsl_kw = Hashtbl.create 37 in
        let acsl_terms = Hashtbl.create 37 in
        List.iter
            (fun (i,t,flag) ->
                Hashtbl.add all_acsl_kw i t;
                if flag then Hashtbl.add acsl_terms i t
            )
            [
                (* come first the ACSL terms *)
                "$true", TTRUE, true;
                "$false", TFALSE, true;
                "$nothing", NOTHING, true;
(*                 "$null", NULL; *)
                "$old", OLD, true;
                "$result", RESULT, true;
                "$const", CONSTANT, true;
                "$valid", VALID, true;
                "$unique", UNIQUE, true;
                (* now come ACSL keywords *)
                "$assert", ASSRT, false;
                "$assigns", ASSGN, false;
                "$assumes", ASSMS, false;
                "$behavior", BHVIR, false;
                "$ensures", ENSRS, false ;
                "$resets", RESET, false;
                "$global", GLOB, false;
                "$loop", LOOP, false;
                "$requires", REQRS, false;
                "$invariant", INVA, false;
                "$is_handler", ISHANDLER, false;
                "$context", CONTEXT, false;
                "$hard", HARDINT_CTX, false;
                "$soft", SOFTINT_CTX, false;
                "$process", PROCESS_CTX, false;
                "$prohibit", PROHIBIT, false;
                "$type", TYPE, false;
                "$except", EXCEPT, false;
                "$head", HEAD, false;
                "$tail", TAIL, false;
                "$continue", CONTINUE, false;
                "$break", BREAK, false;
                "$for", FOR, false;
                "$zero", ZERO, true;
                "$return", RETURN, false;
                "$strong", STRONG_INV, false;
                "$weak", WEAK_INV, false;
                "$read_requires", READ_REQRS, false;
                "$read_ensures", READ_ENSRS, false;
                "$write_requires", WRITE_REQRS, false;
                "$write_ensures", WRITE_ENSRS, false;
            ];
        fun s start_pos ->
            let kw =
                try
                    Hashtbl.find (if Scommon.is_acsl_term_mode () then acsl_terms else all_acsl_kw) s
                with Not_found -> (
                    let current_fn, current_ln, current_cn = !Sexpr.current_pos
                    in
                    let ln, cn = make_local_pos start_pos
                                                current_ln
                                                current_cn
                                                !Sexpr.entered_newline in
                    let error_pos = Sexpr.make_global_pos current_fn
                                                          ln 
                                                          current_cn
                    in
                    let msg = ("Unrecognized ACSL identifier: " ^ s)
                    in
                    raise (Sexpr.Specs_error (msg, Some error_pos)))
            in
            kw
} 

let space = [' ' '\t' '\012' '\r' ]

let digit = ['0'-'9'] 
let octaldigit = ['0'-'7']
let hexadigit = ['a'-'f' 'A'-'F' '0'-'9']
let ident = ['_' 'a'-'z' 'A'-'Z'] 
let ident_num = ['_' 'a'-'z' 'A'-'Z' '0'-'9'] 
let exponent = ['E''e']['+''-']? digit+
let proposition = ['P''p']['+''-']? digit+
let intsuffix = ('u'|'U'|'l'|'L')*
let floatsuffix = ('f'|'F'|'l'|'L')


rule token = parse 
   | space+ {token lexbuf} 
   | '\n' {incr_lineno lexbuf; token lexbuf} 
   (* Comments *)
   | "//" { single_line_comment lexbuf }	  
   | "#{" { current_comment := "#{"; multi_line_comment lexbuf }

   | ident ident_num* 
       { let s = lexeme lexbuf in identifier s }
   | ['$'] ident ident_num* 
       { let s = lexeme lexbuf
         and pos = lexbuf.lex_start_p
         in bs_identifier s pos }

   (* integer constants *)
   | '0' ['x''X'] hexadigit+ intsuffix?
       { CONSTI (lexeme lexbuf) }
   | '0' octaldigit+ intsuffix?
       { CONSTI (lexeme lexbuf) }
   | digit+ intsuffix?
       { CONSTI (lexeme lexbuf) }

   (* floating-point constants *)
   | digit+ exponent floatsuffix?
   | digit* "." digit+ (exponent)? floatsuffix?
   | digit+ "." digit* (exponent)? floatsuffix?
       { CONSTF (lexeme lexbuf) }

   | 'L'? '"' as prelude (([^'"''\n']|"\\\"")+ as content) '"'
       { STRING_LITERAL (prelude.[0] = 'L',content) }

   | ':' {COL}
   | ',' {COM}
   | ';' {SEM}
   | '(' {LPA} 
   | ')' {RPA} 
   | '{' {LBC} 
   | '}' {RBC}
   | '[' {LBK} 
   | ']' {RBK}
   | '.' {DOT}
   | '=' {EQL}

   (*rel/binary op*)
   | "==" {EQ}
   | "!=" {NE}
   | "<=" {LE}
   | ">=" {GE}
   | ">" {GT}
   | "<" {LT}
   | ">>" {GTGT}
   | "<<" {LTLT}
   | "||" {BARBAR}
   | "|" {BAR}
   | "&&"  {AMPAMP}
   | "==>" {IMPL}        
   | "<==>" {EQUI}
   | "^^"  {XOR}
   | "->" {ARW} 
   | "?"  {QES}  
(*    | "++"  {PLSPLS} *)
(*    | "--"  {MNSMNS} *)

   (*unary/binary op*)   
   | '+'  {PLS}
   | '-'  {MNS}
   | '*'  {STR}
   | '/'  {DIV}
(*    | '@'  {ATM}  *)
   | '!'  {BAN}    
   | '&'  {AMP}
   | '%'  {MOD} 
   | '^'  {HAT} 
   | '~'  {TLD}
   | _ as c
          { 
              let current_fn, current_ln, current_cn = !Sexpr.current_pos in
              let start_pos = lexbuf.lex_start_p in
              let ln, cn = make_local_pos start_pos
                                          current_ln
                                          current_cn
                                          !Sexpr.entered_newline in
              let error_pos = Sexpr.make_global_pos current_fn
                                                    ln 
                                                    current_cn
              in
              let msg = ("Unrecognized character: " ^ (String.make 1 c))
              in
              raise (Sexpr.Specs_error (msg, Some error_pos)) }
   | eof  {EOF}
 	
and multi_line_comment = parse
    | "}#"    { current_comment := !current_comment ^ "}#";
                let current_fn, current_ln, current_cn = !Sexpr.current_pos in
                let start_pos = lexbuf.lex_start_p in
                let ln, cn = make_local_pos start_pos
                                            current_ln
                                            current_cn
                                            !Sexpr.entered_newline in
                comment_pos := Sexpr.make_global_pos current_fn
                                                     ln 
                                                     current_cn;
                token lexbuf }
    | _ as c  { current_comment := !current_comment ^ (Char.escaped c);
                multi_line_comment lexbuf }
    | eof     { let msg =  ("Specs comment \""
                            ^ !current_comment
                            ^ "\" is not closed with \"}#\"") in
                raise (Sexpr.Specs_error (msg, Some (!comment_pos))) }

and single_line_comment = parse          
    | '\n' { incr_lineno lexbuf; token lexbuf }
    | _    { single_line_comment lexbuf }
    | eof  { token lexbuf }	
