(*pp camlp4o Camlp4MacroParser.cmo -D${MYMODE} *)

(* utils/logicUtil.ml *)
(* Copyright (C) 2007-2010 Yonezawa Lab, The University of Tokyo *)

(* Written by DzungDK@ANLAB Vietnam *)

(* MEMO: 
     (1) these serve as translators. 
     (2) There are some cases which are not handled yet
*)
open CsisatAst

module Ast = CsisatAst
module Interpolate = CsisatInterpolate
module AstUtil = CsisatAstUtil
module SatPL = CsisatSatPL

let round_coeffs = ref false

let rec print_expr expr = match expr with
  | Constant cst -> CsisatUtils.my_string_of_float cst
  | Variable v -> v
  | Application (sym, lst) -> sym ^ "(" ^ (CsisatUtils.string_list_cat ", " (List.map print_expr lst)) ^")"
  | Sum lst ->  "(" ^ (CsisatUtils.string_list_cat " $PLUS " (List.map print_expr lst)) ^")"
  | Coeff (co, expr) -> (CsisatUtils.my_string_of_float co) ^ " $MUL " ^ (print_expr expr)

let rec print_pred p =
  match p with
  | False -> "false"
  | True -> "true"
  | And lst -> "(" ^ (CsisatUtils.string_list_cat " $AND " (List.map print_pred lst)) ^")"
  | Or lst -> "(" ^ (CsisatUtils.string_list_cat " $OR " (List.map print_pred lst)) ^")"
  | Not p -> "not " ^ print_pred p
  | Eq (e1,e2) -> (print_expr e1) ^ " $EQ " ^ (print_expr e2)
  | Lt (e1,e2) -> (print_expr e1) ^ " $LT " ^ (print_expr e2)
  | Leq (e1,e2) -> (print_expr e1) ^ " $LEQ " ^ (print_expr e2)
  | Atom (External str) -> str
  | Atom (Internal i) -> "atom"^(string_of_int i)

let print p = print_pred p

let rec count_atom_pred_in_pred (pred : predicate) : int = (
    match pred with
    | True
    | False -> 0
    | And preds
    | Or preds -> count_atom_pred_in_preds preds
    | Not pred1 -> count_atom_pred_in_pred pred1
    | Eq _
    | Lt _
    | Leq _
    | Atom _ -> 1
)

and count_atom_pred_in_preds (preds : predicate list) : int = (
    List.fold_left (fun atom_pred_number pred ->
        atom_pred_number + (count_atom_pred_in_pred pred)
    ) 0 preds
)

let get_interpolants (inputs: Ast.predicate list) : Ast.predicate list =
    let conjunction_of (preds : Ast.predicate list) : Ast.predicate =
        Ast.And preds
    in
    let rec get_interpolants1 (inputs1:Ast.predicate list)
                              (inputs2:Ast.predicate list) 
               (to_return: Ast.predicate list) =
    begin
        match inputs2 with
        | [] -> to_return
        | h::tail -> 
                let a = conjunction_of inputs1
                and b = conjunction_of inputs2
                in
                let interpolant = 
                        Interpolate.interpolate_with_proof a b in 
                get_interpolants1 (inputs1 @ [h]) 
                                  tail 
                                  (interpolant::to_return)
    end
    in 
    match inputs with 
    | [] -> []
    | h::t -> get_interpolants1 [h] t []


(* ~~~~~~~~~~~~~~~~~~~~~~~ COMPUTE WEAKEST PRECONDITION ~~~~~~~~~~~~~~~~~~~~~ *)
(* replace var in an expression by an other expression (the other expression  *)
(* is right_expr of an assignment)                                            *)
let rec replace_var_in_expression (expr_in_post_condition : expression) 
    (replacing_var : expression) (expr_to_replace : expression) 
: expression =
begin
	  match expr_in_post_condition with
    | Constant f -> Constant f
	  | Variable _ -> 
        if expr_in_post_condition = replacing_var then expr_to_replace 
        else expr_in_post_condition
    (* 20090626 *)
	  (*| Application ("addrof", _)
	  | Application ("pref", _)
	  | Application ("sref", _)
	  | Application ("srefp", _) ->
	      if expr_in_post_condition = replacing_var then expr_to_replace 
		    else expr_in_post_condition*)
	  (* temporally  process "array" *)
	  | Application ("array", expr_list) -> (*assert false*)
        if expr_in_post_condition = replacing_var then expr_to_replace 
        else begin
            let new_expr_list = ref [] in
            (* replace var in each expr in expr_list *)
            List.iter 
                (fun a_expr -> 
                    let new_expr = 
                        replace_var_in_expression (a_expr) (replacing_var) 
                          (expr_to_replace) 
                    in
                    new_expr_list := !new_expr_list @ [new_expr]
                )
                expr_list;
            (* return *)
            Application ("array", !new_expr_list)
        end
	  | Application (name_of_function_call, expr_list) -> begin
		    let new_expr_list = ref [] in
	      (* replace var in each expr in expr_list *)
		    List.iter 
		        (fun a_expr -> 
		            let new_expr = 
		                replace_var_in_expression (a_expr) (replacing_var) 
	                    (expr_to_replace) 
	              in
		            new_expr_list := !new_expr_list @ [new_expr]
            )
		        expr_list;
	      (* return *)
		    Application (name_of_function_call, !new_expr_list)
	    end
	  | Sum expr_list -> begin
		    let new_expr_list = ref [] in
	      (* replace var in each expr in expr_list *)
		    List.iter 
		        (fun a_expr -> 
		            let new_expr = 
		                replace_var_in_expression (a_expr) (replacing_var) 
	                    (expr_to_replace) 
	              in
		            new_expr_list := !new_expr_list @ [new_expr]
            )
		        expr_list;
	      (* return *)
		    Sum !new_expr_list
	    end
	  | Coeff (c, expr) -> begin
	      let new_expr = 
	          replace_var_in_expression (expr) (replacing_var) (expr_to_replace)
	      in
	      Coeff (c, new_expr)
	    end
end

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
(* replace expression in an predicatee by an other expression (the other      *)
(* expression is right_expr of an assignment)                                 *)
and replace_var_in_predicate (pred_in_post_condition : predicate) 
    (replacing_var : expression) (expr_to_replace : expression) 
: predicate = 
begin
	  match pred_in_post_condition with
	  | True -> True
	  | False -> False
	  | And pred_list -> begin
		    let new_pred_list = ref [] in
		    List.iter 
		        (fun a_pred -> 
			          let new_pred = 
	                  replace_var_in_predicate (a_pred) (replacing_var) 
	                    (expr_to_replace) 
	              in
			          new_pred_list := !new_pred_list @ [new_pred]
            )
		        pred_list;
		      And !new_pred_list
        end
    | Or pred_list -> begin
        let new_pred_list = ref [] in
        List.iter 
            (fun a_pred -> 
                let new_pred : predicate = 
                    replace_var_in_predicate (a_pred) (replacing_var) 
                      (expr_to_replace) 
                in
                new_pred_list := !new_pred_list @ [new_pred]
            )
            pred_list;
        Or !new_pred_list
      end
	  | Not pred -> begin
        let new_pred = 
            replace_var_in_predicate (pred) (replacing_var) (expr_to_replace)
        in
        Not new_pred
      end
	  | Eq (expr1, expr2) -> begin 
		    let new_expr1 : expression = 
            replace_var_in_expression (expr1) (replacing_var) (expr_to_replace) 
		    in
        let new_expr2 : expression = 
            replace_var_in_expression (expr2) (replacing_var) (expr_to_replace) 
        in
		    Eq (new_expr1, new_expr2)
      end
	  | Lt (expr1, expr2) -> begin 
        let new_expr1 : expression = 
            replace_var_in_expression (expr1) (replacing_var) (expr_to_replace) 
        in
        let new_expr2 : expression = 
            replace_var_in_expression (expr2) (replacing_var) (expr_to_replace) 
        in
        Lt (new_expr1, new_expr2)
      end
	  | Leq (expr1, expr2) -> begin 
        let new_expr1 : expression = 
            replace_var_in_expression (expr1) (replacing_var) (expr_to_replace) 
        in
        let new_expr2 : expression = 
            replace_var_in_expression (expr2) (replacing_var) (expr_to_replace) 
        in
        Leq (new_expr1, new_expr2)
      end
	  | Atom i -> Atom i
end

(* ~~~~~~~~~~~~~~~~ compute weakest_precondition ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
let rec get_weakest_precondition (post_condition : predicate)
    (edge : predicate) 
: predicate =
begin
	  (* check type of edge, if edge isn't an assignment or a list of *)
    (* assignments is assert false                                  *)
	  let wp_to_return = 
	      match edge with
		    | Eq (left_expr, right_expr) -> 
		       replace_var_in_predicate (post_condition) (left_expr) (right_expr)
		    | And pred_list ->
          begin 
			      let rev_pred_list = List.rev pred_list in 
			      let weakest_precondition = ref post_condition in
			      List.iter 
		  	        (fun a_pred -> 
			              weakest_precondition :=  
                      get_weakest_precondition (!weakest_precondition) (a_pred)
                )
			          rev_pred_list;
            (* return *)
			      !weakest_precondition
          end
		    | _ -> post_condition;
	  in
	  wp_to_return
end

let rec collect_constant_division (input : expression)
                                  (candidates : (expression * float) list)
                                  : (expression * float) list = (
    match input with
    | Constant _
    | Variable _ -> candidates      
    | Application (sym, args) -> (
        let new_candidates =
            if ((String.compare sym "div") == 0) then (
                assert((List.length args) = 2);
                match args with
                | [divident; divisor] -> (
                    match divisor with
                    | (Constant f) when (f > 0.) -> [(divident, f)]
                    | _ -> [])
                | _ -> assert false)
            else
                []
        in
        List.fold_left (fun candidates expression ->
                            collect_constant_division expression candidates)
                       (candidates @ new_candidates)
                       args)
    | Sum exprs ->
        List.fold_left (fun candidates expression ->
                            collect_constant_division expression candidates)
                       candidates
                       exprs
    | Coeff (_, expr) ->
        collect_constant_division expr candidates)

let rec approximate_division (input : predicate) : predicate = (
    let make_augmented_pred const_div = (
        let divident, divisor = const_div in
        let div_expr =
            Application("div", [divident; Constant divisor])
        in
        let expr1 = Sum [divident; Constant (-.divisor)]
        and expr2 = Coeff (divisor, div_expr)
        in
        let lower_bound_pred = Lt (expr1, expr2)
        and upper_bound_pred = Leq (expr2, divident)
        in
        [lower_bound_pred; upper_bound_pred])
    in
    match input with
    | True
    | False -> input
    | And preds ->
        let new_preds = List.map approximate_division preds in
        And new_preds
    | Or preds ->
        let new_preds = List.map approximate_division preds in
        Or new_preds
    | Not pred ->
        let new_pred = approximate_division pred in
        Not new_pred
    | Eq (lexpr, rexpr) -> (
        let l_candidates = collect_constant_division lexpr []
        and r_candidates = collect_constant_division rexpr []
        in
        let candidates = l_candidates @ r_candidates in
        let augmented_preds =
             List.flatten (List.map make_augmented_pred candidates)
        in
        match augmented_preds with
        | [] -> input
        | _ -> And (input::augmented_preds))
    | Lt (expr1, expr2) -> (
        let candidates1 = collect_constant_division expr1 []
        and candidates2 = collect_constant_division expr2 []
        in
        let candidates = candidates1 @ candidates2 in
        let augmented_preds =
            List.flatten (List.map make_augmented_pred candidates)
        in
        match augmented_preds with
        | [] -> input
        | _ -> And (input::augmented_preds))
    | Leq (expr1, expr2) -> (
        let candidates1 = collect_constant_division expr1 []
        and candidates2 = collect_constant_division expr2 []
        in
        let candidates = candidates1 @ candidates2 in
        let augmented_preds =
            List.flatten (List.map make_augmented_pred candidates)
        in
        match augmented_preds with
        | [] -> input
        | _ -> And (input::augmented_preds))
    | Atom _ -> input)

(* ~~~~~~~~~~~~~~~~~~~ CHECK SATISFIABILITY AND VALIDITY ~~~~~~~~~~~~~~~~~ *)
let is_sat (pred : predicate) : bool = (
      StatisticUtil.update_solver_invocation_number ();
      let pred =
          AstUtil.simplify (AstUtil.normalize (approximate_division pred))
      in
IFDEF DEBUG_MODE THEN
      print_endline ("Checking SAT of : " ^ (print_pred pred));
      let result = SatPL.is_sat (AstUtil.simplify pred) in
      print_endline ("   -->> " ^ (string_of_bool result));
      result
ELSE
      SatPL.is_sat pred
END
)

let is_valid (pred_to_check : predicate) = 
begin
    let new_pred = AstUtil.simplify (AstUtil.normalize (Not pred_to_check)) in
    if is_sat new_pred = false then true
    else false
end

(* ~~~~~~~~~~~~~~~~~~~ CHECK INTERPOLANTS ~~~~~~~~~~~~~~~~~ *)
let check_interpolants (base_preds : predicate list)
                       (interpolants : predicate list)
                       : bool = (
    assert ((List.length base_preds) - (List.length interpolants) = 1);
    let rec check_interpolant a_preds b_preds interpolants =
        assert ((List.length b_preds) > 0);
        match interpolants with
          [] -> true
        | itp :: tails -> (
            let pred_a = And a_preds
            and pred_b = And b_preds
            in
            let a_itp = Or [Not pred_a; itp]
            and b_itp = And [pred_b; itp]
            in
            if not (is_valid a_itp) then
                false
            else (
                if (is_sat b_itp) then
                    false
                else (
                    (* OK, continue checking *)
                    match b_preds with
                      [_] -> true
                    | h :: bs ->
                        check_interpolant (a_preds @ [h]) bs tails)))
    in
    match base_preds with
      []
    | [_] -> true
    | h :: t ->
        check_interpolant [h] t interpolants)

(* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *)
module Logic = struct
  include Ast
  let init: bool -> unit =
      fun (round : bool) -> (
          CsisatGlobal.round := round;
          round_coeffs := round)

  let string_of_predicate: predicate -> string = 
      fun (p: predicate) -> print_pred p
  let string_of_expression : expression -> string = 
      fun (e : expression) -> print_expr e
  
  let get_interpolant (a : predicate) (b : predicate) : predicate = (
      StatisticUtil.update_solver_invocation_number ();
      let new_a = approximate_division a
      and new_b = approximate_division b
      in
      Interpolate.interpolate_with_proof new_a new_b
  )

  let normalize: predicate -> predicate = AstUtil.normalize

  let get_interpolants (preds : predicate list) : predicate list =
      let new_preds = List.map (fun pred ->
                          AstUtil.simplify (normalize (approximate_division pred))
                      ) preds in
      let _ =
IFDEF DEBUG_MODE THEN
      print_endline ("Getting Interpolants of");
      List.iter (fun p -> print_endline ("    * " ^ (string_of_predicate p)))
                new_preds;
ELSE
      ()
END
      in
      let original_results =
          Interpolate.interpolate_with_one_proof new_preds
      in
      let results =
          if (!round_coeffs) then (
              let rounded_results =
                  List.map (CsisatLIUtils.round_coeff) original_results
              in
              if (check_interpolants new_preds rounded_results) then
                  rounded_results
              else
                  original_results)
          else
              original_results
      in
      let _ =
IFDEF DEBUG_MODE THEN
      print_endline ("   >> Results");
      List.iter (fun p -> print_endline ("      * " ^ (string_of_predicate p)))
                results;
      print_endline (" END >>");
ELSE
      ()
END
      in
      results

  let is_sat : predicate -> bool = is_sat
 
  let is_valid: predicate -> bool = is_valid

  let cnf: predicate -> predicate = AstUtil.cnf
  let dnf: predicate -> predicate = AstUtil.dnf

  let approximate_division : predicate -> predicate = approximate_division
  
  let replace_var_in_expression : expression -> expression -> expression -> expression =
      replace_var_in_expression
  let replace_var_in_predicate: predicate -> expression -> expression -> predicate =
      replace_var_in_predicate
  let get_weakest_precondition: predicate -> predicate -> predicate = 
      get_weakest_precondition
  let simplify : predicate -> predicate = AstUtil.simplify 
  let simplify_expr : expression -> expression = AstUtil.simplify_expr
  let remove_lit_clash : predicate -> predicate = AstUtil.remove_lit_clash
  let count_atom_pred_in_pred : predicate -> int = count_atom_pred_in_pred
  let count_atom_pred_in_preds : predicate list -> int = count_atom_pred_in_preds
end
   
