#define vmm_assert(x)
#define guest_assert(x)
#define	print_detail1(...)
#define	print_detail2(...)
#define	print_detail3(...)

#define	AT_PERCPU

#define	profile_begin(x)
#define	profile_end(x)
#define	PROFILE_FUNCTION(x,y)
#define	PROFILE_FUNCTION_END
#define	PROFILE_FUNCTION_END_VOID
#define	PROFILE_NEW(x)

#define	TEMPREF_MAP_PPAGES	tempref_map_ppages
#define	TEMPREF_MAP_PPAGES_IN_RANGE	tempref_map_ppages_in_range
#define	TEMPREF_MAP_P	tempref_map_p
#define	TEMPREF_MAP_G	tempref_map_g

#define TYPE_U(X)	(X ## U)

#define true    1
#define false   0

typedef	int			bool;
typedef unsigned int            uint;
typedef unsigned char           u8;
typedef unsigned short          u16;
typedef unsigned int            u32;
typedef unsigned long long      u64;
typedef signed char             s8;
typedef signed short            s16;
typedef signed int              s32;
typedef signed long long        s64;

typedef u32			page_entry_t;
typedef u32                     size_t;
typedef u32                     addr_t;

typedef struct {
        u16 gs; u16 _pad_gs;
        u16 fs; u16 _pad_fs;
        u16 es; u16 _pad_es;
        u16 ds; u16 _pad_ds;
        u32 edi;
        u32 esi;
        u32 ebp;
        u32 _esp;
        u32 ebx;
        u32 edx;
        u32 ecx;
        u32 eax;
        u32 vector;
        u32 error_code;
        u32 eip;
        u16 cs; u16 _pad_cs;
        u32 eflags;
        u32 esp;
        u16 ss; u16 _pad_ss;
} regs_t;

typedef struct {
        u32 a,b;
} desc_t;

typedef struct {
        u16 limit;
        u32 base;
} desc_ptr_t;

typedef struct {
        u32 offset;
        u16 segment;
} farptr_t;

#define NUM_PDE                 (1024)
#define NUM_PTE                 (1024)

typedef struct {
        page_entry_t entries[NUM_PDE];
} page_directory_t;

typedef struct {
        page_entry_t entries[NUM_PTE];
} page_table_t;

typedef struct {
        uint vtop;
        uint vend;
        char name[16];
} paging_region_t;


typedef struct {
        page_directory_t *dir;
        page_table_t *tables;
        page_table_t *tables_paddr;
        paging_region_t regions[16];
        uint num_of_regions;
} paging_t;

//#define VMM_TOP		TYPE_U(0xfc000000)
#define VMM_TOP			TYPE_U(0xfc00000)	// WRONG VALUE bcause of BUG

#define VMM_RING                (0)
#define GUESTOS_RING            (1)
#define GUESTOS_IOPL            VMM_RING

#define GDT_FIRST_INDEX		(256)
#define GDT_LAST_INDEX		(GDT_FIRST_INDEX + 10)
#define NUM_GDT_ENTRIES		(GDT_LAST_INDEX + 1)

typedef struct {
        desc_t entries[NUM_GDT_ENTRIES];
} gdt_t;

#define NUM_EXCEPTIONS	(256)

typedef struct {
        desc_t entries[NUM_EXCEPTIONS];
} idt_t;

typedef enum {
        GUEST_DOWN = 0,
        GUEST_STARTING,
        GUEST_RUNNING,
        GUEST_HALTING,
} guest_state_t;

#define NUM_CR		(5)

typedef struct range_struct {
        struct range_struct *next;
        uint top;
        uint end;
} range_t;

typedef struct {
        uint gtop;
        uint gend;
        range_t *ppages;
        uint ppage;
        int mbi_type;
} gpmap_region_t;

typedef struct {
        gpmap_region_t regions[64];
        uint num_of_regions;
} gpmap_t;

typedef enum {
        GUEST_OS_UNKNOWN = 0,
        GUEST_OS_LINUX,
} guest_os_type_t;

#define MBI_MAX_CMDLINE			(1600)

typedef unsigned int cpuid_t;
typedef u8 apicid_t;

typedef struct {
        size_t size;
        range_t *pages;
        char string[MBI_MAX_CMDLINE];
} cpu_config_guest_module_t;

typedef struct {
        cpuid_t id;
        apicid_t apicid;
        range_t *percpu_pages;
        struct {
                u32 uses;
                guest_os_type_t os_type;
                bool no_auto_start;
                addr_t load_addr;
                addr_t lower_mem_top;
                addr_t upper_mem_top;
                addr_t upper_mem_end;
                range_t *pages[16];
                gpmap_t gpmap;
                union {
                        struct {
                                cpu_config_guest_module_t kernel_module;
                                cpu_config_guest_module_t initrd_module;
                        } os_linux;
                };
        } guest;
} cpu_config_t;

typedef struct {
        cpu_config_t *config;
#ifndef __ONLY_FOR_MODELCHECKING
        gdt_t gdt;
        idt_t idt;
        struct {
                tss_t body;
                u8 io_bitmap[256*256/8 + 1];
        } tss;
#endif
        paging_t paging;
#ifndef __ONLY_FOR_MODELCHECKING
        DECLARE_BITS_STRUCT(TEMPREF_SIZE/PAGE_SIZE) tempref_bits;
#endif
        struct {
                guest_state_t state;
                addr_t startup;

                u32 cr[NUM_CR];
                desc_ptr_t gdtr;
                desc_ptr_t idtr;
                u16 ldtr;
                u16 taskr;

                u32 halting_error_code;
                u32 halting_vector;
        } guest;

#ifndef __ONLY_FOR_MODELCHECKING
        char stack[STACK_SIZE] __attribute__((aligned(PAGE_SIZE)));
#endif
        page_directory_t page_directory;
        page_table_t page_tables[NUM_PDE];
} cpu_t;

extern cpu_t cpu;

#define CR0_PE		(1 <<  0)
#define CR0_MP		(1 <<  1)
#define CR0_EM		(1 <<  2)
#define CR0_TS		(1 <<  3)
#define CR0_ET		(1 <<  4)
#define CR0_NE		(1 <<  5)
#define CR0_WP		(1 << 16)
#define CR0_AM		(1 << 18)
#define CR0_NW		(1 << 29)
#define CR0_CD		(1 << 30)
#define CR0_PG		(1 << 31)
#define CR0_KEEP_FLAGS (CR0_AM | CR0_WP | CR0_NE | CR0_ET | CR0_TS | CR0_MP)

#define CR3_BASE_MASK	(~((1 << 12) - 1))

#define CR4_VME		(1 <<  0)
#define CR4_PVI		(1 <<  1)
#define CR4_TSD		(1 <<  2)
#define CR4_DE		(1 <<  3)
#define CR4_PSE		(1 <<  4)
#define CR4_PAE		(1 <<  5)
#define CR4_MCE		(1 <<  6)
#define CR4_PGE		(1 <<  7)
#define CR4_PCE		(1 <<  8)
#define CR4_OSFXSR	(1 <<  9)
#define CR4_OSXMMEXCPT	(1 << 10)
#define CR4_KEEP_FLAGS (CR4_TSD | CR4_PCE | CR4_OSFXSR | CR4_OSXMMEXCPT)

#define EFLAGS_CF		(1 <<  0)
#define EFLAGS_PF		(1 <<  2)
#define EFLAGS_ZF		(1 <<  6)
#define EFLAGS_SF		(1 <<  7)
#define EFLAGS_TF		(1 <<  8)
#define EFLAGS_IF		(1 <<  9)
#define EFLAGS_DF		(1 << 10)
#define EFLAGS_OF		(1 << 11)
#define EFLAGS_IOPL_SHIFT	12
#define EFLAGS_IOPL_BITS	(2)
#define EFLAGS_IOPL_MASK	(((1 << EFLAGS_IOPL_BITS) - 1) << EFLAGS_IOPL_SHIFT)
#define EFLAGS_RF		(1 << 16)
#define EFLAGS_VM		(1 << 17)
#define EFLAGS_AC		(1 << 18)
#define EFLAGS_VIF		(1 << 19)
#define EFLAGS_VIP		(1 << 20)
#define EFLAGS_ID		(1 << 21)
#define EFLAGS_PRESERVED	(2)

#define EXCEPTION_DE	(0)
#define EXCEPTION_DB	(1)
#define EXCEPTION_NMI	(2)
#define EXCEPTION_BP	(3)
#define EXCEPTION_OF	(4)
#define EXCEPTION_BR	(5)
#define EXCEPTION_UD	(6)
#define EXCEPTION_NM	(7)
#define EXCEPTION_DF	(8)
#define EXCEPTION_TS	(10)
#define EXCEPTION_NP	(11)
#define EXCEPTION_SS	(12)
#define EXCEPTION_GP	(13)
#define EXCEPTION_PF	(14)
#define EXCEPTION_MF	(16)
#define EXCEPTION_AC	(17)
#define EXCEPTION_MC	(18)
#define EXCEPTION_XF	(19)

#define PF_ERR_P	(1 << 0)
#define PF_ERR_WR	(1 << 1)
#define PF_ERR_US	(1 << 2)
#define PF_ERR_RSVD	(1 << 3)

#define L1_PAGE_SHIFT		(12)
#define L2_PAGE_SHIFT		(22)
#define L1_PAGE_SIZE		(TYPE_U(1) << L1_PAGE_SHIFT)	/* 4KB */
#define L2_PAGE_SIZE		(TYPE_U(1) << L2_PAGE_SHIFT)	/* 4MB */

#define L1_PAGE_OFFSET_MASK	(L1_PAGE_SIZE - 1)
#define L1_PAGE_MASK		(~L1_PAGE_OFFSET_MASK)
#define L2_PAGE_OFFSET_MASK	(L2_PAGE_SIZE - 1)
#define L2_PAGE_MASK		(~L2_PAGE_OFFSET_MASK)

#define PAGE_SHIFT		L1_PAGE_SHIFT
#define	PAGE_SIZE		L1_PAGE_SIZE
#define PAGE_MASK		L1_PAGE_MASK
#define PAGE_OFFSET_MASK	L1_PAGE_OFFSET_MASK

#define PAGE_SHIFT		L1_PAGE_SHIFT
#define PAGE_ENTRY_P		(1 << 0)
#define PAGE_ENTRY_RW		(1 << 1)
#define PAGE_ENTRY_US		(1 << 2)
#define PAGE_ENTRY_PWT		(1 << 3)
#define PAGE_ENTRY_PCD		(1 << 4)
#define PAGE_ENTRY_A		(1 << 5)
#define PAGE_ENTRY_D		(1 << 6)
#define PAGE_ENTRY_PS		(1 << 7)
#define PAGE_ENTRY_G		(1 << 8)
